package com.localtransfer;

import android.animation.ObjectAnimator;
import android.app.Notification;
import android.app.PendingIntent;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.core.app.NotificationManagerCompat;

import com.google.android.material.snackbar.Snackbar;

import java.util.ConcurrentModificationException;

public class Progress {

    private static Notification.Builder bl = new Notification.Builder(Transfer.activity);
    private static NotificationManagerCompat notifiManager = NotificationManagerCompat.from(Transfer.activity);
    private static final LayoutInflater inflater = (LayoutInflater) Transfer.activity.getSystemService(Transfer.activity.LAYOUT_INFLATER_SERVICE);

    private static Intent notificationIntent = new Intent(Transfer.activity, MainActivity.class);
    private static PendingIntent pendingIntent = PendingIntent.getActivity(Transfer.activity,0, notificationIntent, 0);

    public static View root;
    public static boolean fragment_on = false;
    public static boolean app_started = false;

    private static long	savedTimeMillis;

    public static void PreProgress(Transfer tr) {
        Transfer.activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {

                final Drawable image = Transfer.activity.getDrawable(R.drawable.ic_spinner_rotate);

                if(tr.button != null) {
                    int h = image.getIntrinsicHeight();
                    int w = image.getIntrinsicWidth();
                    image.setBounds(0, 0, w, h);
                    tr.button.setCompoundDrawables(tr.button.getCompoundDrawables()[0], null, image, null);
                    ObjectAnimator anim = ObjectAnimator.ofInt(image, "level", 0, 10000);
                    anim.setDuration(1000);
                    anim.setRepeatCount(Animation.INFINITE);
                    anim.setInterpolator(new LinearInterpolator());
                    anim.start();
                }
            }
        });
    }

    public static void ProgressUpdateDelay(Transfer tr) {
        // Run only every second to reduce CPU usage
        if (System.currentTimeMillis() > (savedTimeMillis + 1000)) {
            savedTimeMillis = System.currentTimeMillis();
            ProgressUpdate(tr);
        }
    }

    private static void ProgressUpdate(Transfer tr){
        Transfer.activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {

                if (Progress.app_started && tr.button != null)
                    tr.button.setText(String.format("Download in progress %d%%", tr.percent));

                try {

                    bl.setSmallIcon(R.drawable.ic_upload_and_download_from_the_cloud)
                            .setContentTitle(tr.fileName)
                            .setContentText(String.format("%d%% %s/%s", tr.percent, tr.loadedHuman, tr.fileSizeHuman))
                            .setProgress(100, (int) tr.percent, false)
                            .setContentIntent(pendingIntent);
                    notifiManager.notify(Transfer.NOTIF_SERVICE, bl.build());

                    if (app_started && fragment_on)
                        showProgressFragment(tr);

                } catch (ConcurrentModificationException e) {
                    e.printStackTrace();
                }
            }
        });
    }

    public static void PostProgress(Transfer tr) {
        ProgressUpdate(tr);
        Transfer.activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {

                notifiManager.cancel(Transfer.NOTIF_SERVICE);

                if (!app_started) {
                    bl.setSmallIcon(R.drawable.ic_upload_and_download_from_the_cloud)
                            .setContentTitle(tr.fileName)
                            .setContentText(tr.info)
                            .setProgress(0, 0, false)
                            .setContentIntent(pendingIntent);
                    notifiManager.notify(tr.id, bl.build());
                }

                if (tr.button != null) {
                    final LinearLayout layout = (LinearLayout) tr.button.getParent();
                    layout.findViewById(R.id.file_view).setVisibility(LinearLayout.VISIBLE);
                    layout.findViewById(R.id.file_share).setVisibility(LinearLayout.VISIBLE);
                    tr.button.setVisibility(LinearLayout.GONE);
                    tr.button.setEnabled(true);

                    tr.button.setText(Transfer.activity.getString(R.string.file_download));
                    tr.button.setCompoundDrawables(tr.button.getCompoundDrawables()[0], null, null, null);
                }

                System.out.println(tr.message);
                /*Snackbar.make(Transfer.activity.findViewById(R.id.view_pager), tr.message, Snackbar.LENGTH_LONG)
                        .setAction("Action", null).show();*/
                Toast.makeText(Transfer.activity, tr.message, Toast.LENGTH_SHORT).show();

            }
        });
    }

    public static void showProgressFragment(Transfer tr){

        LinearLayout groot = root.findViewById(R.id.groot);

        final View progress;

        if (groot.findViewById(tr.id) != null) {
            progress = groot.findViewById(tr.id);
        } else {
            progress = inflater.inflate(R.layout.progress, null);
            progress.setId(tr.id);
            Transfer.activity.runOnUiThread(() -> {
                groot.addView(progress, 0);
            });
        }

        ((ProgressBar) progress.findViewById(R.id.progressBar)).setProgress((int) tr.percent);
        ((ImageView) progress.findViewById(R.id.transferType)).setImageResource(tr.drawable);
        ((TextView) progress.findViewById(R.id.progressText)).setText(tr.percent + " %");
        ((TextView) progress.findViewById(R.id.fileName)).setText(tr.fileName);
        ((TextView) progress.findViewById(R.id.fileSize)).setText(String.format("%s/%s", tr.loadedHuman, tr.fileSizeHuman));

        if (tr.state == Transfer.STATE_SUCCESS)
            ((TextView) progress.findViewById(R.id.fileName)).setTextColor(Color.GREEN);
        else if (tr.state == Transfer.STATE_FAILED)
            ((TextView) progress.findViewById(R.id.fileName)).setTextColor(Color.RED);
    }
}
